@@ -1,11 +1,12 @@
 import asyncio
 import logging
 import logging.handlers
+import time
+from contextlib import suppress
 
 from typing import Union
+from .thread_pool import run_in_new_thread
 
-from aiomisc.thread_pool import threaded
-from aiomisc.periodic import PeriodicCallback
 from prettylog import (
     color_formatter,
     create_logging_handler,
@@ -16,8 +17,26 @@ from prettylog import (
 )
 
 
-class AsyncMemoryHandler(logging.handlers.MemoryHandler):
-    flush_async = threaded(logging.handlers.MemoryHandler.flush)
+def _thread_flusher(handler: logging.handlers.MemoryHandler,
+                    flush_interval: Union[float, int],
+                    loop: asyncio.AbstractEventLoop):
+    def has_no_target():
+        return True
+
+    def has_target():
+        return bool(handler.target)
+
+    is_target = has_no_target
+
+    if isinstance(handler, logging.handlers.MemoryHandler):
+        is_target = has_target
+
+    while not loop.is_closed() and is_target():
+        with suppress(Exception):
+            if handler.buffer:
+                handler.flush()
+
+        time.sleep(flush_interval)
 
 
 def wrap_logging_handler(handler: logging.Handler,
@@ -27,12 +46,14 @@ def wrap_logging_handler(handler: logging.Handler,
     loop = loop or asyncio.get_event_loop()
 
     buffered_handler = _wrap_logging_handler(
-        handler=handler, buffer_size=buffer_size,
-        logger_class=AsyncMemoryHandler,
+        handler=handler, buffer_size=buffer_size
     )
 
-    periodic = PeriodicCallback(buffered_handler.flush_async)
-    loop.call_soon_threadsafe(periodic.start, flush_interval, loop)
+    run_in_new_thread(
+        _thread_flusher, args=(
+            buffered_handler, flush_interval, loop
+        ), no_return=True
+    )
 
     return buffered_handler
 
@@ -40,7 +61,7 @@ def wrap_logging_handler(handler: logging.Handler,
 def basic_config(level: int = logging.INFO,
                  log_format: Union[str, LogFormat] = LogFormat.color,
                  buffered: bool = True, buffer_size: int = 1024,
-                 flush_interval: float = 0.2, loop=None):
+                 flush_interval: float = 0.2, loop=None, **kwargs):
 
     if isinstance(level, str):
         level = getattr(logging, level.upper())
@@ -52,7 +73,7 @@ def basic_config(level: int = logging.INFO,
     if isinstance(log_format, str):
         log_format = LogFormat[log_format]
 
-    handler = create_logging_handler(log_format)
+    handler = create_logging_handler(log_format, **kwargs)
 
     if buffered:
         handler = wrap_logging_handler(
@@ -70,8 +91,9 @@ def basic_config(level: int = logging.INFO,
 
 __all__ = (
     'basic_config',
-    'wrap_logging_handler',
-    'JSONLogFormatter',
-    'json_formatter',
     'color_formatter',
+    'json_formatter',
+    'JSONLogFormatter',
+    'LogFormat',
+    'wrap_logging_handler',
 )
