@@ -1,5 +1,6 @@
 import asyncio
 import socket
+from functools import partial
 
 from .base import SimpleServer
 from ..utils import OptionsType, bind_socket
@@ -9,7 +10,7 @@ class TCPServer(SimpleServer):
     PROTO_NAME = 'tcp'
 
     def __init__(self, address: str = None, port: int = None,
-                 options: OptionsType = (), sock=None):
+                 options: OptionsType = (), sock=None, **kwargs):
         if not sock:
             if not (address and port):
                 raise RuntimeError(
@@ -17,7 +18,8 @@ class TCPServer(SimpleServer):
                     '"address" and "port" couple'
                 )
 
-            self.socket = bind_socket(
+            self.make_socket = partial(
+                bind_socket,
                 address=address,
                 port=port,
                 options=options,
@@ -25,19 +27,21 @@ class TCPServer(SimpleServer):
         elif not isinstance(sock, socket.socket):
             raise ValueError('sock must be socket instance')
         else:
-            self.socket = sock
+            self.make_socket = lambda: sock
 
-        super().__init__()
+        self.socket = None
+
+        super().__init__(**kwargs)
 
     async def handle_client(self, reader: asyncio.StreamReader,
                             writer: asyncio.StreamWriter):
         raise NotImplementedError
 
     async def start(self):
+        self.socket = self.make_socket()
         self.server = await asyncio.start_server(
             self.handle_client,
             sock=self.socket,
-            loop=self.loop,
         )
 
     async def stop(self, exc: Exception = None):
